# -*- coding: utf-8 -*-
#
# Snoopdroid
#
# Copyright (C) 2019 Claudio Guarnieri <https://nex.sx>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import requests

from halo import Halo
from terminaltables import AsciiTable

from .ui import info, highlight, green, red

def get_koodous_report(sha256):
    url = "https://api.koodous.com/apks/{}".format(sha256)
    res = requests.get(url)
    return res.json()

def koodous_lookup(packages):
    print(info("Looking up all extracted files on " + highlight("Koodous") + " (www.koodous.com)."))
    print(info("This might take a while..."))
    print("")

    table_data = []
    table_data.append(["Package name", "File path", "Trusted", "Detected", "Rating"])

    with Halo(text="", spinner="bouncingBar") as spinner:
        total_packages = len(packages)
        counter = 0
        for package in packages:
            counter += 1

            spinner.text = "Looking up {} [{}/{}]".format(package.name, counter, total_packages)

            for file in package.files:
                report = get_koodous_report(file["sha256"])

                if "package_name" in report:
                    trusted = "no"
                    if report["trusted"]:
                        trusted = green("yes")

                    detected = "no"
                    if report["detected"]:
                        detected = red("yes")

                    rating = "0"
                    if int(report["rating"]) < 0:
                        rating = red(str(report["rating"]))

                    row = [package.name, file["stored_path"], trusted, detected, rating]
                else:
                    row = [package.name, file["stored_path"], "", "", ""]

                table_data.append(row)

        spinner.succeed("Completed!")

    print("")

    table = AsciiTable(table_data)
    print(table.table)



# -*- coding: utf-8 -*-
#
# Snoopdroid
#
# Copyright (C) 2019 Claudio Guarnieri <https://nex.sx>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import time
import requests

from halo import Halo
from terminaltables import AsciiTable

from .ui import info, highlight, red

def get_virustotal_report(hashes):
    apikey = "233f22e200ca5822bd91103043ccac138b910db79f29af5616a9afe8b6f215ad"
    url = "https://www.virustotal.com/partners/sysinternals/file-reports?apikey={}".format(apikey)

    items = []
    for sha256 in hashes:
        items.append({
            "hash": sha256,
            "image_path": "unknown",
            "creation_datetime": "unknown",
        })
    headers = {"User-Agent": "VirusTotal", "Content-Type": "application/json"}
    res = requests.post(url, headers=headers, json=items)

    if res.status_code == 200:
        report = res.json()
        return report["data"]

    return None

def virustotal_lookup(packages):
    print(info("Looking up all extracted files on " + highlight("VirusTotal") + " (www.virustotal.com)."))
    print("")

    detections = {}

    def virustotal_query(batch):
        report = get_virustotal_report(batch)
        if report:
            for entry in report:
                if entry["hash"] not in detections and entry["found"] == True:
                    detections[entry["hash"]] = entry["detection_ratio"]

    with Halo(text="", spinner="bouncingBar") as spinner:
        batch = []
        for package in packages:
            for file in package.files:
                batch.append(file["sha256"])
                if len(batch) == 25:
                    spinner.text = "Looking up first 25 apps..."

                    virustotal_query(batch)
                    batch = []

        if batch:
            spinner.text = "Looking up remaining files..."
            virustotal_query(batch)

        spinner.succeed("Completed!")

    table_data = []
    table_data.append(["Package name", "File path", "Detections"])

    for package in packages:
        for file in package.files:
            row = [package.name, file["stored_path"],]

            if file["sha256"] in detections:
                detection = detections[file["sha256"]]
                positives = detection.split("/")[0]
                if int(positives) > 0:
                    row.append(red(detection))
                else:
                    row.append(detection)
            else:
                row.append("not found")

            table_data.append(row)

    print("")

    table = AsciiTable(table_data)
    print(table.table)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Snoopdroid
#
# Copyright (C) 2019 Claudio Guarnieri <https://nex.sx>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import os
import sys
import argparse
import datetime

from .ui import info, logo
from .acquisition import Acquisition
from .virustotal import virustotal_lookup
from .koodous import koodous_lookup

def main():
    parser = argparse.ArgumentParser(description="Extract information from Android device")
    parser.add_argument("--storage", default=os.getcwd(), help="Specify a different base folder to store the acquisition")
    parser.add_argument("--limit", default=None, help="Set a limit to the number of packages to extract (mainly for debug purposes)")
    parser.add_argument("--all-apks", action="store_true", help="Extract all packages installed on the phone, even those marked as safe")
    parser.add_argument("--virustotal", action="store_true", help="Check packages on VirusTotal")
    parser.add_argument("--koodous", action="store_true", help="Check packages on Koodous")
    parser.add_argument("--all-checks", action="store_true", help="Run all available checks")
    parser.add_argument("--from-file", default=None, help="Instead of acquiring from phone, load an existing packages.json file for lookups (mainly for debug purposes)")
    args = parser.parse_args()

    logo()

    try:
        if args.from_file:
            acq = Acquisition.fromJSON(args.from_file)
        else:
            # TODO: Need to come up with a better folder name.
            acq_folder = datetime.datetime.now().isoformat().split(".")[0].replace(":", "")
            storage_folder = os.path.join(args.storage, acq_folder)

            acq = Acquisition(storage_folder=storage_folder,
                all_apks=args.all_apks, limit=args.limit)
            acq.run()
        
        packages = acq.packages

        if len(packages) == 0:
            return

        if args.virustotal or args.all_checks:
            virustotal_lookup(packages)

        print("")

        if args.koodous or args.all_checks:
            koodous_lookup(packages)
    except KeyboardInterrupt:
        print("")
        sys.exit(-1)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# Snoopdroid
# Copyright (C) 2019  Claudio Guarnieri
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import os
from setuptools import setup

from snoopdroid.constants import __version__

description = "Extract all apks from an Android device and check for malicious apps"

this_directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(this_directory, "README.md"), encoding="utf-8") as handle:
    long_description = handle.read()

requires = (
    "rsa",
    "tqdm",
    "halo",
    "requests",
    "terminaltables",
    "adb",
)

def get_package_data(package):
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}

setup(
    name="snoopdroid",
    version=__version__,
    author="Claudio Guarnieri",
    author_email="nex@nex.sx",
    description=description,
    long_description=long_description,

    scripts=["bin/snoopdroid",],
    install_requires=requires,
    packages=["snoopdroid",],
    package_data=get_package_data('snoopdroid'),
    include_package_data=True,

    keywords="security android",
    license="GPLv3",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Topic :: Security",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: Android",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS",
        "Environment :: Console",
    ],
)
